/*
 *  Public Key layer for parsing key files and structures
 *
 *  Copyright (C) 2006-2015, ARM Limited, All Rights Reserved
 *  SPDX-License-Identifier: Apache-2.0
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may
 *  not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 *  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  This file is part of mbed TLS (https://tls.mbed.org)
 */



#include "eccderkeyparse.h"
#include "asn1.h"
#include "oid.h"

#include <string.h>
#include <stdlib.h>

/*
 * EC public key is an EC point
 *
 * The caller is responsible for clearing the structure upon failure if
 * desired. Take care to pass along the possible ECP_FEATURE_UNAVAILABLE
 * return code of mbedtls_ecp_point_read_binary() and leave p in a usable state.
 */
static int pk_get_ecpubkey( unsigned char **p, const unsigned char *end,
                            mbedtls_ecp_keypair *key )
{
    int ret;

    if( ( ret = mbedtls_ecp_point_read_binary( &key->grp, &key->Q,
                    (const unsigned char *) *p, end - *p ) ) == 0 )
    {
        ret = mbedtls_ecp_check_pubkey( &key->grp, &key->Q );
    }

    /*
     * We know mbedtls_ecp_point_read_binary consumed all bytes or failed
     */
    *p = (unsigned char *) end;

    return( ret );
}


/*
 * Use EC parameters to initialise an EC group
 *
 * ECParameters ::= CHOICE {
 *   namedCurve         OBJECT IDENTIFIER
 *   specifiedCurve     SpecifiedECDomain -- = SEQUENCE { ... }
 *   -- implicitCurve   NULL
 */
static int pk_use_ecparams( const mbedtls_asn1_buf *params, mbedtls_ecp_group *grp )
{
    int ret;
    mbedtls_ecp_group_id grp_id;

    if( params->tag == MBEDTLS_ASN1_OID )
    {
        if( mbedtls_oid_get_ec_grp( params, &grp_id ) != 0 )	//  sawyer
            return( MBEDTLS_ERR_PK_UNKNOWN_NAMED_CURVE );
    }
    else
    {
//#if defined(MBEDTLS_PK_PARSE_EC_EXTENDED)
//        if( ( ret = pk_group_id_from_specified( params, &grp_id ) ) != 0 )
//            return( ret );
//#else
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT );
//#endif
    }

    /*
     * grp may already be initilialized; if so, make sure IDs match
     */
    if( grp->id != MBEDTLS_ECP_DP_NONE && grp->id != grp_id )
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT );

    if( ( ret = mbedtls_ecp_group_load( grp, grp_id ) ) != 0 )
        return( ret );

    return( 0 );
}


static int pk_get_ecparams( unsigned char **p, const unsigned char *end,
                            mbedtls_asn1_buf *params )
{
    int ret;

    /* Tag may be either OID or SEQUENCE */
    params->tag = **p;
    if( params->tag != MBEDTLS_ASN1_OID
#if defined(MBEDTLS_PK_PARSE_EC_EXTENDED)
            && params->tag != ( MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE )
#endif
            )
    {
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT +
                MBEDTLS_ERR_ASN1_UNEXPECTED_TAG );
    }

    if( ( ret = mbedtls_asn1_get_tag( p, end, &params->len, params->tag ) ) != 0 )
    {
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT + ret );
    }

    params->p = *p;
    *p += params->len;

    if( *p != end )
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT +
                MBEDTLS_ERR_ASN1_LENGTH_MISMATCH );

    return( 0 );
}

/* Get a PK algorithm identifier
 *
 *  AlgorithmIdentifier  ::=  SEQUENCE  {
 *       algorithm               OBJECT IDENTIFIER,
 *       parameters              ANY DEFINED BY algorithm OPTIONAL  }
 */
static int pk_get_pk_alg( unsigned char **p,
                          const unsigned char *end,
                          mbedtls_pk_type_t *pk_alg, mbedtls_asn1_buf *params )
{
    int ret;
    mbedtls_asn1_buf alg_oid;

    memset( params, 0, sizeof(mbedtls_asn1_buf) );

    if( ( ret = mbedtls_asn1_get_alg( p, end, &alg_oid, params ) ) != 0 )
        return( MBEDTLS_ERR_PK_INVALID_ALG + ret );

    if( mbedtls_oid_get_pk_alg( &alg_oid, pk_alg ) != 0 )
        return( MBEDTLS_ERR_PK_UNKNOWN_PK_ALG );

    /*
     * No parameters with RSA (only for EC)
     */
    if( *pk_alg == MBEDTLS_PK_RSA &&
            ( ( params->tag != MBEDTLS_ASN1_NULL && params->tag != 0 ) ||
                params->len != 0 ) )
    {
        return( MBEDTLS_ERR_PK_INVALID_ALG );
    }

    return( 0 );
}

/*
 *  SubjectPublicKeyInfo  ::=  SEQUENCE  {
 *       algorithm            AlgorithmIdentifier,
 *       subjectPublicKey     BIT STRING }
 */
static int mbedtls_pk_parse_subpubkey( unsigned char **p, const unsigned char *end,
                        mbedtls_ecp_keypair *eck )
{
    int ret;
    size_t len;
    mbedtls_asn1_buf alg_params;
    mbedtls_pk_type_t pk_alg = MBEDTLS_PK_NONE;

    if( ( ret = mbedtls_asn1_get_tag( p, end, &len,
                    MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE ) ) != 0 )
    {
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT + ret );
    }

    end = *p + len;

    if( ( ret = pk_get_pk_alg( p, end, &pk_alg, &alg_params ) ) != 0 )
        return( ret );

    if( ( ret = mbedtls_asn1_get_bitstring_null( p, end, &len ) ) != 0 )
        return( MBEDTLS_ERR_PK_INVALID_PUBKEY + ret );

    if( *p + len != end )
        return( MBEDTLS_ERR_PK_INVALID_PUBKEY +
                MBEDTLS_ERR_ASN1_LENGTH_MISMATCH );

        ret = pk_use_ecparams( &alg_params, &eck->grp );
        if( ret == 0 )
            ret = pk_get_ecpubkey( p, end, eck );

    if( ret == 0 && *p != end )
        ret = MBEDTLS_ERR_PK_INVALID_PUBKEY
              MBEDTLS_ERR_ASN1_LENGTH_MISMATCH;

    return( ret );
}


int mbedtls_parse_ec_der_key(unsigned char *derkey, size_t keylen, mbedtls_ecp_keypair *eck)
{
	  int ret;
    int version, pubkey_done;
    size_t len;
    mbedtls_asn1_buf params;
    unsigned char *p = (unsigned char *) derkey;
    unsigned char *end = p + keylen;
    unsigned char *end2;

    /*
     * RFC 5915, or SEC1 Appendix C.4
     *
     * ECPrivateKey ::= SEQUENCE {
     *      version        INTEGER { ecPrivkeyVer1(1) } (ecPrivkeyVer1),
     *      privateKey     OCTET STRING,
     *      parameters [0] ECParameters {{ NamedCurve }} OPTIONAL,
     *      publicKey  [1] BIT STRING OPTIONAL
     *    }
     */
    if( ( ret = mbedtls_asn1_get_tag( &p, end, &len,
            MBEDTLS_ASN1_CONSTRUCTED | MBEDTLS_ASN1_SEQUENCE ) ) != 0 )
    {
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT + ret );
    }

    end = p + len;

    if( ( ret = mbedtls_asn1_get_int( &p, end, &version ) ) != 0 )
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT + ret );

    if( version != 1 )
        return( MBEDTLS_ERR_PK_KEY_INVALID_VERSION );

    if( ( ret = mbedtls_asn1_get_tag( &p, end, &len, MBEDTLS_ASN1_OCTET_STRING ) ) != 0 )
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT + ret );

    if( ( ret = mbedtls_mpi_read_binary( &eck->d, p, len ) ) != 0 )
    {
        mbedtls_ecp_keypair_free( eck );
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT + ret );
    }

    p += len;

    pubkey_done = 0;
    if( p != end )
    {
        /*
         * Is 'parameters' present?
         */
        if( ( ret = mbedtls_asn1_get_tag( &p, end, &len,
                        MBEDTLS_ASN1_CONTEXT_SPECIFIC | MBEDTLS_ASN1_CONSTRUCTED | 0 ) ) == 0 )
        {
            if( ( ret = pk_get_ecparams( &p, p + len, &params) ) != 0 ||
                ( ret = pk_use_ecparams( &params, &eck->grp )  ) != 0 )
            {
                mbedtls_ecp_keypair_free( eck );
                return( ret );
            }
        }
        else if( ret != MBEDTLS_ERR_ASN1_UNEXPECTED_TAG )
        {
            mbedtls_ecp_keypair_free( eck );
            return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT + ret );
        }

        /*
         * Is 'publickey' present? If not, or if we can't read it (eg because it
         * is compressed), create it from the private key.
         */
        if( ( ret = mbedtls_asn1_get_tag( &p, end, &len,
                        MBEDTLS_ASN1_CONTEXT_SPECIFIC | MBEDTLS_ASN1_CONSTRUCTED | 1 ) ) == 0 )
        {
            end2 = p + len;

            if( ( ret = mbedtls_asn1_get_bitstring_null( &p, end2, &len ) ) != 0 )
                return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT + ret );

            if( p + len != end2 )
                return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT +
                        MBEDTLS_ERR_ASN1_LENGTH_MISMATCH );

            if( ( ret = pk_get_ecpubkey( &p, end2, eck ) ) == 0 )
                pubkey_done = 1;
            else
            {
                /*
                 * The only acceptable failure mode of pk_get_ecpubkey() above
                 * is if the point format is not recognized.
                 */
                if( ret != MBEDTLS_ERR_ECP_FEATURE_UNAVAILABLE )
                    return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT );
            }
        }
        else if( ret != MBEDTLS_ERR_ASN1_UNEXPECTED_TAG )
        {
            mbedtls_ecp_keypair_free( eck );
            return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT + ret );
        }
    }

    if( ! pubkey_done &&
        ( ret = mbedtls_ecp_mul( &eck->grp, &eck->Q, &eck->d, &eck->grp.G,
                                                      NULL, NULL ) ) != 0 )
    {
        mbedtls_ecp_keypair_free( eck );
        return( MBEDTLS_ERR_PK_KEY_INVALID_FORMAT + ret );
    }

    if( ( ret = mbedtls_ecp_check_privkey( &eck->grp, &eck->d ) ) != 0 )
    {
        mbedtls_ecp_keypair_free( eck );
        return( ret );
    }

    return( 0 );
	
}

int mbedtls_parse_ec_der_pub_key(unsigned char *derkey, size_t keylen, mbedtls_ecp_keypair *eck)
{
    int ret;
    unsigned char *p;
    p = (unsigned char *) derkey;

    ret = mbedtls_pk_parse_subpubkey( &p, p + keylen, eck );
    return( ret );
}	


#if defined(MBEDTLS_ECCDERKEYPARSE_SELF_TEST)


unsigned char derkeyarray[] = {	
        //  this is prim256v1(secp256r1)
        0x30,0x77,0x02,0x01,0x01,0x04,0x20,0xF2,0xF4,0x30,0x3F,0xBB,0x94,0x40,0x9C,0x02,
	0x14,0x26,0xFA,0x9B,0x7B,0x28,0x4B,0x73,0x06,0xF4,0xA1,0xE5,0x13,0x7F,0x81,0xA6,
	0x45,0x38,0xBE,0xD6,0x19,0x0D,0xDD,0xA0,0x0A,0x06,0x08,0x2A,0x86,0x48,0xCE,0x3D,
	0x03,0x01,0x07,0xA1,0x44,0x03,0x42,0x00,0x04,0x7F,0xE5,0xB2,0x58,0x5E,0xB0,0x0E,
	0x29,0xD4,0xFE,0x8F,0x93,0xDD,0xC7,0xB0,0x2B,0x6A,0xE8,0xBF,0xC7,0x3A,0x96,0x33,
	0xAC,0x37,0xD5,0xAE,0xFA,0x6F,0x12,0x0B,0x9D,0x82,0xAB,0x6E,0x62,0x66,0xBE,0x44,
	0xEC,0x59,0x43,0x27,0xA1,0x39,0x07,0xAB,0x8E,0xE5,0x1D,0x2D,0x5C,0xF2,0x01,0x8B,
	0xBE,0x91,0x84,0x65,0xFB,0x67,0x79,0x5C,0xE1
  
//        // this is from secp256k1-key.der
//	0x30,0x74,0x02,0x01,0x01,0x04,0x20,0x8A,0xCF,0x57,0xE2,0x74,0xB0,0xF3,0x31,0x44,
//	0x34,0xBB,0x69,0x36,0x9D,0xDC,0x75,0xE7,0x81,0x23,0x55,0x4C,0xF9,0x22,0xAF,0xFB,
//	0x98,0xA2,0x3D,0x73,0xC3,0xEA,0x13,0xA0,0x07,0x06,0x05,0x2B,0x81,0x04,0x00,0x0A,
//	0xA1,0x44,0x03,0x42,0x00,0x04,0xA7,0x36,0x7D,0xCB,0xFE,0x3C,0x50,0xF2,0x28,0x39,
//	0xCF,0x9D,0xBC,0x04,0xAA,0x4B,0xA3,0x6F,0xE3,0x0D,0xBE,0x36,0x9F,0x2B,0x26,0x74,
//	0xB5,0x73,0xE8,0x21,0xF6,0xB9,0x9E,0xC7,0x52,0x15,0x84,0x37,0x43,0x89,0x4D,0x45,
//	0x19,0x87,0x6A,0xA0,0x9B,0xA6,0x20,0x58,0xD2,0xC0,0x3C,0x59,0xF4,0xC9,0x02,0x30,
//	0x4B,0xAD,0x4A,0xA4,0x1C,0x8E
};
unsigned int derkeysize = sizeof(derkeyarray);

unsigned char derpubkeyarray[] = {	
        // this is prim256v1(secp256r1)
        0x30,0x59,0x30,0x13,0x06,0x07,0x2A,0x86,0x48,0xCE,0x3D,0x02,0x01,0x06,0x08,0x2A,
	0x86,0x48,0xCE,0x3D,0x03,0x01,0x07,0x03,0x42,0x00,0x04,0x7F,0xE5,0xB2,0x58,0x5E,
	0xB0,0x0E,0x29,0xD4,0xFE,0x8F,0x93,0xDD,0xC7,0xB0,0x2B,0x6A,0xE8,0xBF,0xC7,0x3A,
	0x96,0x33,0xAC,0x37,0xD5,0xAE,0xFA,0x6F,0x12,0x0B,0x9D,0x82,0xAB,0x6E,0x62,0x66,
	0xBE,0x44,0xEC,0x59,0x43,0x27,0xA1,0x39,0x07,0xAB,0x8E,0xE5,0x1D,0x2D,0x5C,0xF2,
	0x01,0x8B,0xBE,0x91,0x84,0x65,0xFB,0x67,0x79,0x5C,0xE1
        // this is from secp256k1-pub.der
//	0x30,0x56,0x30,0x10,0x06,0x07,0x2A,0x86,0x48,0xCE,0x3D,0x02,0x01,0x06,0x05,0x2B,
//	0x81,0x04,0x00,0x0A,0x03,0x42,0x00,0x04,0xA7,0x36,0x7D,0xCB,0xFE,0x3C,0x50,0xF2,
//	0x28,0x39,0xCF,0x9D,0xBC,0x04,0xAA,0x4B,0xA3,0x6F,0xE3,0x0D,0xBE,0x36,0x9F,0x2B,
//	0x26,0x74,0xB5,0x73,0xE8,0x21,0xF6,0xB9,0x9E,0xC7,0x52,0x15,0x84,0x37,0x43,0x89,
//	0x4D,0x45,0x19,0x87,0x6A,0xA0,0x9B,0xA6,0x20,0x58,0xD2,0xC0,0x3C,0x59,0xF4,0xC9,
//	0x02,0x30,0x4B,0xAD,0x4A,0xA4,0x1C,0x8E
};
unsigned int derpubkeysize = sizeof(derpubkeyarray);


/*
 * Checkup routine
 */
int mbedtls_eccderkeyparse_self_test( int verbose )
{
	int ret = 0;
        mbedtls_ecp_keypair keypair;
	mbedtls_ecp_keypair_init( &keypair );
	mbedtls_printf("ECC_DER_KEY_PARSE: ");
        ret = mbedtls_parse_ec_der_key(derkeyarray, derkeysize, &keypair);
	if( (ret == 0 ) && (verbose != 0) ){
		mbedtls_printf( "passed\r\n" );
	} else if(verbose != 0){        //  need to check the curve is supported ?
                mbedtls_ecp_keypair_free( &keypair );
                mbedtls_printf( "failed\r\n" );
                return ret;
        }
	mbedtls_ecp_keypair_free( &keypair );
	
	mbedtls_ecp_keypair_init( &keypair );
	mbedtls_printf("ECC_DER_PUB_KEY_PARSE: ");
	ret = mbedtls_parse_ec_der_pub_key(derpubkeyarray, derpubkeysize, &keypair);
        if( (ret == 0) && (verbose != 0) ){
		mbedtls_printf( "passed\r\n" );
	} else if(verbose != 0){
                mbedtls_printf( "failed\r\n" );
        }
	mbedtls_ecp_keypair_free( &keypair );
	
	return ret;
}

#endif // MBEDTLS_ECCDERKEYPARSE_SELF_TEST

